{ $HDR$}
{**********************************************************************}
{ Unit archived using Team Coherence                                   }
{ Team Coherence is Copyright 2002 by Quality Software Components      }
{                                                                      }
{ For further information / comments, visit our WEB site at            }
{ http://www.TeamCoherence.com                                         }
{**********************************************************************}
{}
{ $Log:  13207: MdRulers.pas 
{
{   Rev 1.1    19.5.2008 . 07:27:32  Daniel
}
{
{   Rev 1.0    11.5.2007 . 00:12:34  Daniel
}
{
{   Rev 1.0    21/08/2005 22:03:56  Daniel
}
{
{   Rev 1.0    09.2.2003 . 00:30:38  daniel
}

  {**********************************************}
  {*                                            *}
  {*  Microdor VCL Extensions                   *}
  {*                                            *}
  {*  Copyright (c) 2001 by Microdor Ltd.       *}
  {*                                            *}
  {**********************************************}

  {
Version 1.00 Release 2001

  Metrics Text
    At left or bottom part appears metrics text such as 'cm' or 'inch'
    Font for the metrics text
    Positioning the text in the ruler
  Supports Track line in the ruler
    Enable and disable the track
    Pen for the Track
  Supports 3 kinds of Lines
    Setting the positioning in the ruller
  Numbers
    Vertical and Horizontal numbers
    Font for numbers
  Range
    Range from Start position to End position
    Range from Start position to the Track position


:
1.            .   
 Kind     .
2.

  }
//------------------------------------------------------------------------------
unit DSCustomGraphicRuler;
//------------------------------------------------------------------------------
interface
//------------------------------------------------------------------------------
uses
  Windows, Messages, Classes, Graphics, Controls, Forms, LineStep;
//------------------------------------------------------------------------------
type
//------------------------------------------------------------------------------
  TDSCustomGraphicRuler = class;

  TRulerTrack = class(TPersistent)
  private
    fOwner: TDSCustomGraphicRuler;
    fPen: TPen; // Specifies the Pen which will be used for drawing the track line
    fVisible: Boolean; // Specify wether or not the track line appears in the ruler
    fPosition: Integer; // Currunt position of the track line in the ruler control
    fMin: Integer; // The track position can not be less then that
    fMax: Integer; // The track position can not be higher then that
    fBorderOffset: Integer; // Specifies the offset of the ruler border where the track line starts to be drawn
    fTrackLen: Integer; // Specifies the length of the track line
    fOnChange: TNotifyEvent;
    procedure setPen(const Value: TPen);
    procedure setVisible(const Value: Boolean);
    procedure setPosition(const Value: Integer);
    procedure setMin(const Value: Integer);
    procedure setMax(const Value: Integer);
    procedure setBorderOffset(const Value: Integer);
    procedure setTrackLen(const Value: Integer);
    procedure PenChange(Sender: TObject);
  protected
    procedure DoChange;
  public
    constructor Create(aOwner: TDSCustomGraphicRuler);
    destructor Destroy; override;
    procedure DrawTrack(Canvas: TCanvas);
    property Owner: TDSCustomGraphicRuler read FOwner;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  published
    property Pen: TPen read fPen write setPen;
    property Visible: Boolean read fVisible write setVisible default True;
    property Position: Integer read fPosition write setPosition default 0;
    property Min: Integer read fMin write setMin default 0;
    property Max: Integer read fMax write setMax default 0;
    property BorderOffset: Integer read fBorderOffset write setBorderOffset default 1;
    property TrackLen: Integer read fTrackLen write setTrackLen default 21;
  end;
//------------------------------------------------------------------------------
  { Marks an area of the ruler as selected range from a Start to End position or to TrackPos}
  TRulerRange = class(TPersistent)
  private
    fOwner: TDSCustomGraphicRuler;
    fPen: TPen;
    fEnabled: Boolean;
    fPosStart: Integer;
    fPosEnd: Integer;
    fToTrack: Boolean;
    fOnChange: TNotifyEvent;
    procedure setEnabled(const Value: Boolean);
    procedure setPosStart(const Value: Integer);
    procedure setPosEnd(const Value: Integer);
    procedure setToTrack(Value: Boolean);
    procedure setPen(const Value: TPen);
    procedure PenChange(Sender: TObject);
  protected
    procedure doChange; virtual;
  public
    constructor Create(aOwner: TDSCustomGraphicRuler);
    destructor Destroy; override;
    procedure DrawRange(Canvas: TCanvas);
    property Owner: TDSCustomGraphicRuler read fOwner;
  published
    property Pen: TPen read FPen write setPen;
    property Enabled: Boolean read FEnabled write setEnabled default False;
    property PosStart: Integer read FPosStart write setPosStart default 0;
    property PosEnd: Integer read FPosEnd write setPosEnd default 0;
    property ToTrack: Boolean read FToTrack write setToTrack default False;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;
  //----------------------------------------------------------------------------

  TRulerKind = (rkLeft, rkTop, rkRight, rkBottom);

  TRulerBevel = TBevelCut;

  TRulerKindState = class
  protected
    procedure FloatTextOut(X, Y: Integer; Val: Double);
  public
    Ruler: TDSCustomGraphicRuler;
    RealPos: Double;
    EndPos: Integer;
    NumberPos: Integer;
    Cnv: TCanvas;
    LargeS, LargeE: Integer;
    MiddleS, MiddleE: Integer;
    SmallS, SmallE: Integer;
    Interval: Double;
    hFnt: HFONT;
    procedure Prepare; virtual; abstract;
    procedure DrawBig; virtual; abstract;
    procedure DrawMiddle; virtual; abstract;
    procedure DrawSmall; virtual; abstract;
  end;

//------------------------------------------------------------------------------
  TVerticalKind = class(TRulerKindState)
  public
    procedure DrawBig; override;
    procedure DrawMiddle; override;
    procedure DrawSmall; override;
  end;

//------------------------------------------------------------------------------
  THorizontalKind = class(TRulerKindState)
  public
    procedure DrawBig; override;
    procedure DrawMiddle; override;
    procedure DrawSmall; override;
  end;

//------------------------------------------------------------------------------
  TRulerMeasure = (rmCustom, rmMm, rmCm, rmDm, rmM, rmInch, rmPixel);  

  TDSCustomGraphicRuler = class(TCustomControl)
  private
    fScreenBitmap: TBitmap;
    fKind: TRulerKind;
    fKindState: TRulerKindState;

    fMeasureText: string;
    fMeasureVisible: Boolean;

    fTrack: TRulerTrack;
    fRange: TRulerRange;

    fOnPaintLogical: TNotifyEvent;
    fNumFont: TFont;
    fSteps: TRulerSteps;
    fColorTo: TColor;
    fStartPos: double;
    fScaleFactor: double;
    fMinPixelsPerBigStep: Integer;
    fIdentEnd: Integer;
    fIdentStart: Integer;
    fOpposite: Boolean;
    fMeasureFactor: Double;
    fMeasure: TRulerMeasure;
    
    procedure setKind(const Value: TRulerKind);
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure setTrack(const Value: TRulerTrack);
    procedure setRange(const Value: TRulerRange);
    procedure setNumFont(const Value: TFont);
    procedure setSteps(const Value: TRulerSteps);
    procedure setColorTo(const Value: TColor);
    procedure DrawGradientH(aCanvas: TCanvas; FromColor, ToColor: TColor; aRect: TRect);
    procedure DrawGradientV(aCanvas: TCanvas; FromColor, ToColor: TColor; aRect: TRect);
    procedure drawDiagonalGradient(aCanvas: TCanvas; FromColor, ToColor: TColor; aRect: TRect);
    procedure setStartPos(const Value: double);
    procedure setScaleFactor(const Value: double);
    procedure setMinPixelsPerBigStep(const Value: Integer);
    procedure setIdentEnd(const Value: Integer);
    procedure setIdentStart(const Value: Integer);
    procedure setOpposite(const Value: Boolean);
    procedure setMeasure(const Value: TRulerMeasure);
    procedure setMeasureText(const Value: string);
    procedure setMeasureFactor(const Value: Double);
    procedure setMeasureVisible(const Value: Boolean);
  protected
    procedure Paint; override;
    procedure setParent(AParent: TWinControl); override;
    procedure StepChange(Sender: TObject);
    procedure PaintParallel3Line(Cnv: TCanvas; aEndPos: Integer); virtual;
    procedure DrawMeasureText();
    function getBigStepInterval(aValue: Double): Double;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    procedure DrawScreenBitmap; virtual;
    procedure LogicalPaint; virtual;
    procedure TrackChange(Sender: TObject); virtual;
    procedure RangeChange(Sender: TObject); virtual;
    function FormatToScreen(aValue: Double): Integer;
    function ScreenToFormat(aValue: Integer): Double;
    function PixelToFormat(aValue: Integer): Double;
    function getTotalMeasureFactor: Double;
  protected
    property StartPos: double read fStartPos write setStartPos;
    property ScaleFactor: double read fScaleFactor write setScaleFactor;
    property MinPixelsPerBigStep: Integer read fMinPixelsPerBigStep write setMinPixelsPerBigStep;
    property IdentStart: Integer read fIdentStart write setIdentStart;
    property IdentEnd: Integer read fIdentEnd write setIdentEnd;
    property Opposite: Boolean read fOpposite write setOpposite default False;
    property MeasureText: string read fMeasureText write setMeasureText;
    property MeasureFactor: Double read fMeasureFactor write setMeasureFactor;
    property MeasureVisible: Boolean read fMeasureVisible write setMeasureVisible default True;
    property Measure: TRulerMeasure read fMeasure write setMeasure;


    property OnPaintLogical: TNotifyEvent read fOnPaintLogical write fOnPaintLogical;
    property Kind: TRulerKind read fKind write setKind default rkTop;
    property Track: TRulerTrack read fTrack write setTrack;
    property Range: TRulerRange read fRange write setRange;
    property NumFont: TFont read fNumFont write setNumFont;
    property Color default clBtnFace;
    property ColorTo: TColor read fColorTo write setColorTo;
    property Steps: TRulerSteps read fSteps write setSteps;
  end;//TDSCustomGraphicRuler
//------------------------------------------------------------------------------
  TDrawLineProc = procedure (Canvas: TCanvas; Val1, Val2, Val3: Integer) of object;
//------------------------------------------------------------------------------
implementation
//------------------------------------------------------------------------------
uses
  Math,
  Types,
  SysUtils;
//------------------------------------------------------------------------------
procedure CopyBitmapOnCanvas(Source: TBitmap; Dest: TCanvas; SourceRect, DestRect: TRect;
   IsTransparent: Boolean; TransparentColor: TColor);
begin
  if IsTransparent then
  begin
    Dest.Brush.Style := bsClear;
    Dest.BrushCopy(DestRect, Source, SourceRect, TransparentColor);
  end
  else
    Dest.CopyRect(DestRect, Source.Canvas, SourceRect);
end;
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// TRulerRange // 
//------------------------------------------------------------------------------
constructor TRulerRange.Create(AOwner: TDSCustomGraphicRuler);
begin
  fOwner := AOwner;
  fPen := TPen.Create;
  fPen.Color := $00C8643C;
  fPen.Mode := pmNotXor;
  fPen.OnChange := PenChange;
  fEnabled := False;
  fPosStart := 0;
  fPosEnd := 0;
  fToTrack := False;
end;

destructor TRulerRange.Destroy;
begin
  fPen.Free;
  inherited Destroy;
end;

procedure TRulerRange.SetPen(const Value: TPen);
begin
  FPen.Assign(Value);
end;

procedure TRulerRange.SetEnabled(const Value: Boolean);
begin
  if Value <> FEnabled then begin
    FEnabled := Value;
    DoChange;
  end
end;

procedure TRulerRange.SetPosStart(const Value: Integer);
begin
  if Value <> FPosStart then begin
    FPosStart := Value;
    DoChange;
  end;
end;

procedure TRulerRange.SetPosEnd(const Value: Integer);
begin
  if Value <> FPosEnd then begin
    FPosEnd := Value;
    DoChange;
  end;
end;

procedure TRulerRange.SetToTrack(Value: Boolean);
begin
  if Value and (Owner = nil) then Value := False; // The track position must be taken from the Owner
  if Value <> FToTrack then begin
    FToTrack := Value;
    DoChange;
  end;
end;

procedure TRulerRange.doChange;
begin
  if Assigned(fOnChange) then
    fOnChange(Self);
end;

procedure TRulerRange.DrawRange(Canvas: TCanvas);
var
  i, RangeStart, RangeEnd: Integer;
  TmpPen: TPen;
begin
  if not Enabled or (Owner = nil) then Exit;
  RangeStart := PosStart;
  if ToTrack then begin
    RangeEnd := Owner.Track.Position;
  end else begin
    RangeEnd := PosEnd;
  end;
  TmpPen := Canvas.Pen;
  Canvas.Pen := Pen;
  try
    if (Owner.Kind = rkTop) or (Owner.Kind = rkBottom) then begin
      for i := 0 to 1 do
        Canvas.Polyline([Point(RangeStart, 13 + i), Point(RangeEnd, 13 + i)]);
    end else begin
      for i := 0 to 1 do
        Canvas.Polyline([Point(13 + i, RangeStart), Point(13 + i, RangeEnd)]);
    end;
  finally
    Canvas.Pen := TmpPen;
  end;
end;

procedure TRulerRange.PenChange(Sender: TObject);
begin
  DoChange;
end;

{ TRulerTrack }

constructor TRulerTrack.Create(aOwner: TDSCustomGraphicRuler);
begin
  fOwner := aOwner;
  fPen := TPen.Create;
  fPen.Mode := pmNotXor;
  fPen.OnChange := PenChange;
  fVisible := True;
  fPosition := 0;
  fMin := 0;
  fMax := 0;
  fBorderOffset := 1;
  fTrackLen := 21;
end;

destructor TRulerTrack.Destroy;
begin
  FPen.Free;
  inherited Destroy;
end;

procedure TRulerTrack.SetPen(const Value: TPen);
begin
  FPen.Assign(Value);
end;

procedure TRulerTrack.SetVisible(const Value: Boolean);
begin
  if Value <> FVisible then begin
    FVisible := Value;
    DoChange;
  end;
end;

procedure TRulerTrack.SetPosition(const Value: Integer);
begin
  if Value <> FPosition then begin
    FPosition := Value;
    DoChange;
  end;
end;

procedure TRulerTrack.SetMin(const Value: Integer);
begin
  if Value <> FMin then begin
    FMin := Value;
    DoChange;
  end;
end;

procedure TRulerTrack.SetMax(const Value: Integer);
begin
  if Value <> FMax then begin
    FMax := Value;
    DoChange;
  end;
end;

procedure TRulerTrack.SetBorderOffset(const Value: Integer);
begin
  if Value <> FBorderOffset then begin
    FBorderOffset := Value;
    DoChange;
  end;
end;

procedure TRulerTrack.SetTrackLen(const Value: Integer);
begin
  if Value <> FTrackLen then begin
    FTrackLen := Value;
    DoChange;
  end;
end;

procedure TRulerTrack.PenChange(Sender: TObject);
begin
  DoChange;
end;

procedure TRulerTrack.DrawTrack(Canvas: TCanvas);
var
  TmpPen: TPen;
  StPt, EndPt: TPoint;
begin
  if not Visible or (Owner = nil) then
    Exit;


  TmpPen := Canvas.Pen;
  try
//    Canvas.Pen := Pen;
    Canvas.Pen.Assign(Pen);
    if (Owner.Kind = rkTop) or (Owner.Kind = rkBottom) then begin
      StPt := Point(Position + Owner.IdentStart, BorderOffset);
      EndPt := Point(Position+ Owner.IdentStart, BorderOffset + TrackLen);
    end else begin
      StPt := Point(BorderOffset, Position + Owner.IdentStart);
      EndPt := Point(BorderOffset + TrackLen, Position + Owner.IdentStart);
    end;
    Canvas.Polyline([StPt, EndPt]);
//    Polyline(Canvas.Handle, [StPt, EndPt], 2)
  finally
//    Canvas.Pen := TmpPen;
    Canvas.Pen.Assign(TmpPen);
  end;
end;

procedure TRulerTrack.DoChange;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

//------------------------------------------------------------------------------
{ TDSCustomGraphicRuler }
constructor TDSCustomGraphicRuler.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fScreenBitmap := TBitmap.Create;
  fScreenBitmap.Width := Width;
  fScreenBitmap.Height := Height;
  fScreenBitmap.PixelFormat := pf24bit;
  fKind := rkTop;
  fKindState := THorizontalKind.Create;
  fKindState.Ruler := Self;
  fOpposite := False;

  fSteps := TRulerSteps.Create();
  fSteps.onStepChange := StepChange;
  Color := $00FFDEDE;
  fColorTo := $00F5B789;
  Width := 257;
  Height := 21;
  fTrack := TRulerTrack.Create(Self);
  fTrack.OnChange := TrackChange;
  fRange := TRulerRange.Create(Self);
  fRange.OnChange := RangeChange;
  fRange.ToTrack := True;
  fNumFont := TFont.Create;
  fNumFont.Height := -9;
  fNumFont.Color := $00C8643C;
  
  fMeasureText := 'cm';
  fMeasureVisible := True;
  fMeasure := rmCm;
  fMeasureFactor := 2.54;

  fStartPos := 0;
  fScaleFactor := 5;
  fMinPixelsPerBigStep := 50;
end;

destructor TDSCustomGraphicRuler.Destroy;
begin
  fScreenBitmap.Free;
  fTrack.Free;
  fRange.Free;
  fNumFont.Free;
  fSteps.Free;
  FreeAndNil(fKindState);
  inherited Destroy;
end;

procedure TDSCustomGraphicRuler.DrawScreenBitmap;
begin
  CopyBitmapOnCanvas(fScreenBitmap, Canvas, ClientRect, ClientRect, False, clNone);
  Track.DrawTrack(Canvas);
  if Range.Enabled then
    Range.DrawRange(Canvas);
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.Paint;
begin
  DrawScreenBitmap;
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.setParent(aParent: TWinControl);

  function FindAligned(aKind: TRulerKind): TDSCustomGraphicRuler;
  var
    i: Integer;
  begin
    Result := nil;
    for i := 0 to Parent.ControlCount - 1 do
      if Parent.Controls[i] is TDSCustomGraphicRuler then
        if Parent.Controls[i] <> Self then
          if TDSCustomGraphicRuler(Parent.Controls[i]).Kind = aKind then begin
            Result := TDSCustomGraphicRuler(Parent.Controls[i]);
            Break;
          end;
  end;

var
  topRuler: TDSCustomGraphicRuler;
begin
  if (csLoading in ComponentState) or not (csDesigning in ComponentState) then
  begin
    inherited SetParent(aParent);
    Exit;
  end;

  inherited setParent(aParent);

  if not Assigned(Parent) then
    Exit;

  topRuler := FindAligned(rkTop);
  if topRuler = nil then begin
    Kind := rkTop;
    Align := alTop;
  end
  else
  if FindAligned(rkLeft) = nil then begin
    Kind := rkLeft;
    Align := alLeft;
    topRuler.IdentStart := Width;
  end
  else
  if FindAligned(rkRight) = nil then begin
    Kind := rkRight;
    Align := alRight;
  end  
  else
  if FindAligned(rkBottom) = nil then begin
    Kind := rkBottom;
    Align := alBottom;
  end;

end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.DrawGradientH(aCanvas: TCanvas; FromColor, ToColor: TColor; aRect: TRect);
var
  StartR, StartG, StartB: Double;
  RStepR, RStepG, RStepB: Double;
  i: Integer;
  vWidth: Integer;
begin
  FromColor := ColorToRGB(FromColor);
  ToColor   := ColorToRGB(ToColor);

  StartR := (FromColor and $000000FF);
  StartG := (FromColor and $0000FF00) shr 8;
  StartB := (FromColor and $00FF0000) shr 16;

  vWidth := aRect.Right - aRect.Left;
  RStepR := ((ToColor and $000000FF) - StartR) / vWidth;
  RStepG := (((ToColor and $0000FF00) shr 8) - StartG) / vWidth;
  RStepB := (((ToColor and $00FF0000) shr 16) - StartB) / vWidth;

  aCanvas.Brush.Style := bsClear;
  for i := aRect.Left to aRect.Right do
  begin
    aCanvas.Pen.Color := (Round(StartR) or (Round(StartG) shl 8) or (Round(StartB) shl 16));
    aCanvas.MoveTo(i, aRect.Top);
    aCanvas.LineTo(i, aRect.Bottom);
    StartR := StartR + RStepR;
    StartG := StartG + RStepG;
    StartB := StartB + RStepB;
  end;
end;//DrawGradientH
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.DrawGradientV(aCanvas: TCanvas; FromColor, ToColor: TColor; aRect: TRect);
var
  StartR, StartG, StartB: Double;
  RStepR, RStepG, RStepB: Double;
  i: Integer;
  vHeight: Integer;
begin
  FromColor := ColorToRGB(FromColor);
  ToColor   := ColorToRGB(ToColor);

  StartR := (FromColor and $000000FF);
  StartG := (FromColor and $0000FF00) shr 8;
  StartB := (FromColor and $00FF0000) shr 16;

  vHeight := aRect.Bottom - aRect.Top;
  RStepR := ((ToColor and $000000FF) - StartR) / vHeight;
  RStepG := (((ToColor and $0000FF00) shr 8) - StartG) / vHeight;
  RStepB := (((ToColor and $00FF0000) shr 16) - StartB) / vHeight;

  aCanvas.Brush.Style := bsClear;
  for i := aRect.Top to aRect.Bottom do
  begin
    aCanvas.Pen.Color := (Round(StartR) or (Round(StartG) shl 8) or (Round(StartB) shl 16));
    aCanvas.MoveTo(aRect.Left, i);
    aCanvas.LineTo(aRect.Right, i);
    StartR := StartR + RStepR;
    StartG := StartG + RStepG;
    StartB := StartB + RStepB;
  end;
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.drawDiagonalGradient(aCanvas: TCanvas; FromColor,
  ToColor: TColor; aRect: TRect);
var
  StartR, StartG, StartB: Double;
  RStepR, RStepG, RStepB: Double;
  i, j: Integer;
  vWidth: Integer;
begin
  FromColor := ColorToRGB(FromColor);
  ToColor   := ColorToRGB(ToColor);

  StartR := (FromColor and $000000FF);
  StartG := (FromColor and $0000FF00) shr 8;
  StartB := (FromColor and $00FF0000) shr 16;

  vWidth := aRect.Right - aRect.Left;
  RStepR := ((ToColor and $000000FF) - StartR) / vWidth;
  RStepG := (((ToColor and $0000FF00) shr 8) - StartG) / vWidth;
  RStepB := (((ToColor and $00FF0000) shr 16) - StartB) / vWidth;

  aCanvas.Brush.Style := bsClear;
  j := aRect.Top;
  for i := aRect.Left to aRect.Right do
  begin
    aCanvas.Pen.Color := (Round(StartR) or (Round(StartG) shl 8) or (Round(StartB) shl 16));
    aCanvas.MoveTo(i, j);
    aCanvas.LineTo(i, aRect.Bottom);
    StartR := StartR + RStepR;
    StartG := StartG + RStepG;
    StartB := StartB + RStepB;
    Inc(j)
  end;

  StartR := (FromColor and $000000FF);
  StartG := (FromColor and $0000FF00) shr 8;
  StartB := (FromColor and $00FF0000) shr 16;
  j := aRect.Left;
  for i := aRect.Top to aRect.Bottom do
  begin
    aCanvas.Pen.Color := (Round(StartR) or (Round(StartG) shl 8) or (Round(StartB) shl 16));
    aCanvas.MoveTo(j, i);
    aCanvas.LineTo(aRect.Right, i);
    StartR := StartR + RStepR;
    StartG := StartG + RStepG;
    StartB := StartB + RStepB;
    Inc(j)
  end;
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.LogicalPaint;
var
  rct: TRect;
begin
  rct := GetClientRect;
  if (Kind in [rkTop, rkBottom]) then begin
    DrawGradientV(fScreenBitmap.Canvas, Color, ColorTo, ClientRect);
    PaintParallel3Line(fScreenBitmap.Canvas, ClientWidth - 10);
  end
  else begin
    DrawGradientH(fScreenBitmap.Canvas, Color, ColorTo, ClientRect);
    PaintParallel3Line(fScreenBitmap.Canvas, ClientHeight - 10);
  end;

  DrawMeasureText();

  if IdentStart > 0 then begin
    rct.Right := IdentStart;
    drawDiagonalGradient(fScreenBitmap.Canvas, Color, ColorTo, rct);
  end;

  if Assigned(fOnPaintLogical) then
    fOnPaintLogical(Self);
    
  DrawScreenBitmap;
end;//LogicalPaint
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.WMSize(var Message: TWMSize);
begin
  inherited;
  fScreenBitmap.Width := Message.Width;
  fScreenBitmap.Height := Message.Height;
  LogicalPaint;
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  Message.Result := 1;
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.setKind(const Value: TRulerKind);
var
  InvertIt: Boolean;
begin
  if Value <> fKind then begin

    if fKind in [rkLeft, rkRight] then
      InvertIt := (Value = rkTop) or (Value = rkBottom)
    else
      InvertIt := (Value = rkLeft) or (Value = rkRight);

    fKind := Value;

    fKindState.Free;
    if fKind in [rkLeft, rkRight] then
      fKindState := TVerticalKind.Create
    else
      fKindState := THorizontalKind.Create;
    fKindState.Ruler := Self;

    if not (csLoading in ComponentState) then
      if InvertIt then
        setBounds(Left, Top, Height, Width);
        
    LogicalPaint;
  end;
end;
//------------------------------------------------------------------------------
{ vInterval -        . }
procedure TDSCustomGraphicRuler.PaintParallel3Line(Cnv: TCanvas; aEndPos: Integer);
var
  minUnits: Double;

  // Returns the start position
  function getStartPosition: Double;
  begin
    if StartPos < 0 then
      Result := minUnits
    else
      Result := 0;
    Result := Trunc(StartPos / minUnits) * minUnits - Result;
  end;

  function CreateNumbersFont: HFONT;
  var
    LogFont: TLogFont;
  begin
    with LogFont do begin
      lfHeight := NumFont.Height; // 1 - lfHeight
      lfWidth := 0; // 2 - lfWidth
      if (Kind = rkTop) or (Kind = rkBottom) then // 3 - lfEscapement
        lfEscapement := 0 //0 gradusa
      else
        lfEscapement := 900; //90 gradusa
      lfOrientation := lfEscapement; // 4 - lfOrientation
      if fsBold in NumFont.Style then // 5 - lfWeight
        lfWeight := FW_BOLD
      else
        lfWeight := FW_NORMAL;
      lfItalic := Byte(fsItalic in NumFont.Style); // 6 - lfItalic
      lfUnderline := Byte(fsUnderline in NumFont.Style); // 7 - lfUnderline
      lfStrikeOut := Byte(fsStrikeOut in NumFont.Style); // 8 - lfStrikeOut
      lfCharSet := Byte(NumFont.Charset); // 9 - lfCharSet

      lfOutPrecision := OUT_TT_ONLY_PRECIS; // 10 - lfOutPrecision
      lfClipPrecision := CLIP_DEFAULT_PRECIS; // 11 - lfClipPrecision
      LogFont.lfQuality := PROOF_QUALITY; // 12 - lfQuality
      case NumFont.Pitch of // 13 - lfPitchAndFamily
        fpVariable: lfPitchAndFamily := FF_DONTCARE or VARIABLE_PITCH;
        fpFixed: lfPitchAndFamily := FF_DONTCARE or FIXED_PITCH;
      else
        lfPitchAndFamily := FF_DONTCARE or DEFAULT_PITCH;
      end;
      StrPCopy(lfFaceName, NumFont.Name); // 14 - lfFaceName
    end;
      //
    Result := CreateFontIndirect(LogFont);
  end;


  procedure PrepareStepsTop;
  begin
    fKindState.LargeE := ClientHeight - Steps.LargeStep.Offset;
    fKindState.LargeS := fKindState.LargeE - Steps.LargeStep.Length;
    fKindState.MiddleE := ClientHeight - Steps.MiddleStep.Offset;
    fKindState.MiddleS := fKindState.MiddleE - Steps.MiddleStep.Length;
    fKindState.SmallE := ClientHeight - Steps.SmallStep.Offset;
    fKindState.SmallS := fKindState.SmallE - Steps.SmallStep.Length;
    fKindState.NumberPos := 1;
  end;

  procedure PrepareStepsBottom;
  begin
    fKindState.LargeS := Steps.LargeStep.Offset;
    fKindState.LargeE := fKindState.LargeS + Steps.LargeStep.Length;
    fKindState.MiddleS := Steps.MiddleStep.Offset;
    fKindState.MiddleE := fKindState.MiddleS + Steps.MiddleStep.Length;
    fKindState.SmallS := Steps.SmallStep.Offset;
    fKindState.SmallE := fKindState.SmallS + Steps.SmallStep.Length;
    fKindState.NumberPos := ClientHeight - Abs(NumFont.Height) - 3;
  end;

  procedure PrepareStepsLeft();
  begin
    fKindState.LargeE := ClientWidth - Steps.LargeStep.Offset;
    fKindState.LargeS := fKindState.LargeE - Steps.LargeStep.Length;
    fKindState.MiddleE := ClientWidth - Steps.LargeStep.Offset;
    fKindState.MiddleS := fKindState.MiddleE - Steps.MiddleStep.Length;
    fKindState.SmallE := ClientWidth - Steps.MiddleStep.Offset;
    fKindState.SmallS := fKindState.SmallE - Steps.SmallStep.Length;
    fKindState.NumberPos := 1;
  end;

  procedure PrepareStepsRight();
  begin
    fKindState.LargeS := Steps.LargeStep.Offset;
    fKindState.LargeE := fKindState.LargeS + Steps.LargeStep.Length;
    fKindState.MiddleS := Steps.MiddleStep.Offset;
    fKindState.MiddleE := fKindState.MiddleS + Steps.MiddleStep.Length;
    fKindState.SmallS := Steps.SmallStep.Offset;
    fKindState.SmallE := fKindState.SmallS + Steps.SmallStep.Length;
    fKindState.NumberPos := ClientWidth - Abs(NumFont.Height) - 3;
  end;

begin
  fKindState.EndPos := aEndPos;
  fKindState.Cnv := Cnv;
  

  minUnits := getBigStepInterval(PixelToFormat(MinPixelsPerBigStep));

  fKindState.Interval := minUnits / 10;
  if (Opposite) then
    fKindState.Interval := -fKindState.Interval;

  fKindState.RealPos := getStartPosition();

  fKindState.hFnt := CreateNumbersFont;
  try
    case (Kind) of
      rkTop: PrepareStepsTop();
      rkBottom: PrepareStepsBottom();
      rkLeft: PrepareStepsLeft();
      rkRight: PrepareStepsRight();
    else
      raise Exception.Create('Invalid ruler kind!');
    end;
    fKindState.DrawBig();
  finally
    DeleteObject(fKindState.hFnt);
  end;
end;//PaintParallel3Line
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.CMColorChanged(var Message: TMessage);
begin
  inherited;
  if Visible and (Parent <> nil) then
    LogicalPaint;
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.setTrack(const Value: TRulerTrack);
begin
  fTrack.Assign(Value);
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.TrackChange(Sender: TObject);
begin
//  if not (csLoading in ComponentState) then
//    LogicalPaint;
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.SetRange(const Value: TRulerRange);
begin
  fRange.Assign(Value);
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.RangeChange(Sender: TObject);
begin
//  if not (csLoading in ComponentState) then
//    LogicalPaint;
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.SetNumFont(const Value: TFont);
begin
  fNumFont.Assign(Value);
  LogicalPaint;
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.setSteps(const Value: TRulerSteps);
begin
  fSteps.Assign(Value);
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.StepChange(Sender: TObject);
begin
  LogicalPaint;
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.setColorTo(const Value: TColor);
begin
  fColorTo := Value;
  LogicalPaint;
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.setStartPos(const Value: double);
begin
  fStartPos := Value;
  LogicalPaint();
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.setScaleFactor(const Value: double);
begin
  fScaleFactor := Value;
  LogicalPaint();
end;
//------------------------------------------------------------------------------
function TDSCustomGraphicRuler.getTotalMeasureFactor: Double;
begin
  if Abs(MeasureFactor) < 0.000001 then
    Result := ScaleFactor
  else
    Result := ScaleFactor /(Screen.PixelsPerInch / MeasureFactor);
end;
//------------------------------------------------------------------------------
function TDSCustomGraphicRuler.FormatToScreen(aValue: Double): Integer;
begin
  if Opposite then
    Result := IdentStart - Trunc((aValue - StartPos) / getTotalMeasureFactor)
  else
    Result := IdentStart + Trunc((aValue - StartPos) / getTotalMeasureFactor);
end;
//------------------------------------------------------------------------------
function TDSCustomGraphicRuler.ScreenToFormat(aValue: Integer): Double;
begin
  if Opposite then
    Result := StartPos - ((aValue - IdentStart) * getTotalMeasureFactor)
  else
    Result := StartPos + ((aValue - IdentStart) * getTotalMeasureFactor);
end;
//------------------------------------------------------------------------------
function TDSCustomGraphicRuler.PixelToFormat(aValue: Integer): Double;
begin
  Result := aValue * getTotalMeasureFactor;
end;
//------------------------------------------------------------------------------
// Draws the text for the measure
procedure TDSCustomGraphicRuler.DrawMeasureText;
var
  LogFont: TLogFont;
  hFnt, hFntPrev: HFONT;
begin
  with LogFont do
  begin
    lfHeight := Font.Height; // 1 - lfHeight
    lfWidth := 0; // 2 - lfWidth
    if (Kind = rkTop) or (Kind = rkBottom) then // 3 - lfEscapement
      lfEscapement := 0 //0 gradusa
    else
      lfEscapement := 900; //90 gradusa
    lfOrientation := lfEscapement; // 4 - lfOrientation
    if fsBold in Font.Style then // 5 - lfWeight
      lfWeight := FW_BOLD
    else
      lfWeight := FW_NORMAL;
    lfItalic := Byte(fsItalic in Font.Style); // 6 - lfItalic
    lfUnderline := Byte(fsUnderline in Font.Style); // 7 - lfUnderline
    lfStrikeOut := Byte(fsStrikeOut in Font.Style); // 8 - lfStrikeOut
    lfCharSet := Byte(Font.Charset); // 9 - lfCharSet

    lfOutPrecision := OUT_TT_ONLY_PRECIS; // 10 - lfOutPrecision
    lfClipPrecision := CLIP_DEFAULT_PRECIS; // 11 - lfClipPrecision
//      LogFont.lfQuality := PROOF_QUALITY; // 12 - lfQuality
    LogFont.lfQuality := ANTIALIASED_QUALITY; // 12 - lfQuality

    case Font.Pitch of // 13 - lfPitchAndFamily
      fpVariable: lfPitchAndFamily := FF_DONTCARE or VARIABLE_PITCH;
      fpFixed: lfPitchAndFamily := FF_DONTCARE or FIXED_PITCH;
    else
      lfPitchAndFamily := FF_DONTCARE or DEFAULT_PITCH;
    end;
    StrPCopy(lfFaceName, Font.Name); // lfFaceName
  end;

  if MeasureVisible then begin
    hFnt := CreateFontIndirect(LogFont);
    try
      hFntPrev := SelectObject(fScreenBitmap.Canvas.Handle, hFnt);
      setTextColor(fScreenBitmap.Canvas.Handle, ColorToRGB(Font.Color));
      if (Kind = rkTop) or (Kind = rkBottom) then
        fScreenBitmap.Canvas.TextOut(Width - fScreenBitmap.Canvas.TextWidth(fMeasureText) - 3, -1, fMeasureText)
      else
        fScreenBitmap.Canvas.TextOut(1, Height - 3, fMeasureText);
      SelectObject(fScreenBitmap.Canvas.Handle, hFntPrev);
    finally
      DeleteObject(hFnt);
    end;
  end;  
end;
//------------------------------------------------------------------------------
// Calculates the interval for showing the big steps in real area values
// It can be ...0.001, 0.01, 0.1, 1, 10, 100 etc.
// aValue - Minimum length for big step in real area values
function TDSCustomGraphicRuler.getBigStepInterval(aValue: Double): Double;
begin
  Result := 1;

  if (Result < aValue) then begin
    while (Result < aValue) do
      Result := Result * 10;
  end
  else begin
    while (Result > aValue) do
      Result := Result / 10;
    Result := Result * 10;
  end;
end;//getBigStepInterval
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.setMinPixelsPerBigStep(const Value: Integer);
begin
  fMinPixelsPerBigStep := Value;
  LogicalPaint();
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.setIdentEnd(const Value: Integer);
begin
  fIdentEnd := Value;
  LogicalPaint();
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.setIdentStart(const Value: Integer);
begin
  fIdentStart := Value;
  LogicalPaint();
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.setOpposite(const Value: Boolean);
begin
  fOpposite := Value;
  LogicalPaint();
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.setMeasureText(const Value: string);
begin
  fMeasureText := Value;
  LogicalPaint();
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.setMeasureFactor(const Value: Double);
begin
  fMeasureFactor := Value;
  LogicalPaint();
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.setMeasureVisible(const Value: Boolean);
begin
  fMeasureVisible := Value;
  LogicalPaint();
end;
//------------------------------------------------------------------------------
procedure TDSCustomGraphicRuler.setMeasure(const Value: TRulerMeasure);
begin
  fMeasure := Value;
  case fMeasure of
  rmMm: fMeasureFactor := 25.4;
  rmCm: fMeasureFactor := 2.54;
  rmDm: fMeasureFactor := 0.254;
  rmM: fMeasureFactor := 0.0254;
  rmInch: fMeasureFactor := 1;
  rmPixel: fMeasureFactor := 0;
  end;
  LogicalPaint();
end;
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
{ TVerticalKind }
//------------------------------------------------------------------------------
procedure TVerticalKind.DrawBig;
var
  CurPos: Integer;
begin
  CurPos := Ruler.FormatToScreen(RealPos);
  while CurPos <= EndPos do begin
    if (CurPos > Ruler.IdentStart) then begin
      Cnv.Pen.Color := Ruler.Steps.LargeStep.Color;
      Cnv.MoveTo(LargeS, CurPos);
      Cnv.LineTo(LargeE, CurPos);
      if ((CurPos + 15) < EndPos) and (CurPos - Ruler.IdentStart - 14 > 0) then
        FloatTextOut(NumberPos, CurPos + 5, RealPos);
    end;

    RealPos := RealPos + Interval;

    DrawSmall;
    DrawMiddle;
    
    CurPos := Ruler.FormatToScreen(RealPos);
  end;
end;

procedure TVerticalKind.DrawMiddle;
var
  i, CurPos: Integer;
begin
  for i := 0 to 0 do begin
    CurPos := Ruler.FormatToScreen(RealPos);
    if CurPos > EndPos then
      Break;
    if (CurPos > Ruler.IdentStart) then begin
      Cnv.Pen.Color := Ruler.Steps.MiddleStep.Color;
      Cnv.MoveTo(MiddleS, CurPos);
      Cnv.LineTo(MiddleE, CurPos);
    end;

    RealPos := RealPos + Interval;
    DrawSmall;
  end;
end;

procedure TVerticalKind.DrawSmall;
var
  i, CurPos: Integer;
begin
  for i := 0 to 3 do begin
    CurPos := Ruler.FormatToScreen(RealPos);
    if CurPos > EndPos then
      Break;

    if (CurPos > Ruler.IdentStart) then begin
      Cnv.Pen.Color := Ruler.Steps.SmallStep.Color;
      
      Cnv.MoveTo(SmallS, CurPos); // Dif
      Cnv.LineTo(SmallE, CurPos); // Dif
    end;

    RealPos := RealPos + Interval;
  end;
end;

{ THorizontalKind }

procedure THorizontalKind.DrawBig;
var
  CurPos: Integer;
begin
  CurPos := Ruler.FormatToScreen(RealPos);
  while CurPos <= EndPos do begin
    if CurPos > Ruler.IdentStart then begin
      Cnv.Pen.Color := Ruler.Steps.LargeStep.Color;
      Cnv.MoveTo(CurPos, LargeS);
      Cnv.LineTo(CurPos, LargeE);
      if ((CurPos + 20) < EndPos) and (CurPos - Ruler.IdentStart - 14 > 0) then
        FloatTextOut(CurPos - 5, NumberPos, RealPos);
    end;
    
    RealPos := RealPos + Interval;

    DrawSmall;
    DrawMiddle;

    CurPos := Ruler.FormatToScreen(RealPos);
  end;
end;

procedure THorizontalKind.DrawMiddle;
var
  i, CurPos: Integer;
begin
  for i := 0 to 0 do begin
    CurPos := Ruler.FormatToScreen(RealPos);
    if CurPos > EndPos then
      Break;
    if (CurPos > Ruler.IdentStart) then begin
      Cnv.Pen.Color := Ruler.Steps.MiddleStep.Color;
      Cnv.MoveTo(CurPos, MiddleS);
      Cnv.LineTo(CurPos, MiddleE);
    end;

    RealPos := RealPos + Interval;
    DrawSmall;
  end;
end;

procedure THorizontalKind.DrawSmall;
var
  i, CurPos: Integer;
begin
  for i := 0 to 3 do begin
    CurPos := Ruler.FormatToScreen(RealPos);
    if CurPos > EndPos then
      Break;

    if (CurPos > Ruler.IdentStart) then begin
      Cnv.Pen.Color := Ruler.Steps.SmallStep.Color;

      Cnv.MoveTo(CurPos, SmallS); // Dif
      Cnv.LineTo(CurPos, SmallE); // Dif
    end;

    RealPos := RealPos + Interval;
  end;
end;

{ TRulerKindState }
//------------------------------------------------------------------------------
procedure TRulerKindState.FloatTextOut(X, Y: Integer; Val: Double);
var
  hFntPrev: HFONT;

  function getRoundValue(): TRoundToRange;
  var
    vBigInterval: Double; 
  begin
    Result := 0;
    vBigInterval := Abs(Interval) * 10;
    
    if (vBigInterval < 1) then begin
      while (vBigInterval < 1) do begin
        Result := Result - 1;
        vBigInterval := vBigInterval * 10;
      end;
    end;
  end;

begin
  hFntPrev := SelectObject(Cnv.Handle, hFnt);
  try
    Val := RoundTo(Val, getRoundValue);
    Cnv.Brush.Style := bsClear;
    setTextColor(Cnv.Handle, ColorToRGB(Ruler.NumFont.Color));
    Cnv.TextOut(X, Y, FloatToStr(Val))
  finally
    SelectObject(Cnv.Handle, hFntPrev);
  end;
end;//FloatTextOut
//------------------------------------------------------------------------------

end.
