unit LineStep;

interface

uses
  Classes, Graphics;

type

  TRulerSteps = class;

  TLineStep = class(TPersistent)
  private
    fOwner: TRulerSteps;
    fOffset: Integer;
    fLength: Integer;
    fColor: TColor;
    procedure setColor(const Value: TColor);
  protected
    procedure setOffset(const Value: Integer);
    procedure setLength(const Value: Integer);
    procedure doChange;
    procedure Change; virtual;
  public
    constructor Create(aOwner: TRulerSteps); virtual;
    destructor Destroy; override;
    property Owner: TRulerSteps read fOwner;
  published  
    property Offset: Integer read fOffset write setOffset;
    property Length: Integer read fLength write setLength;
    property Color: TColor read fColor write setColor;
  end;//TLineStep
//------------------------------------------------------------------------------

  TRulerSteps = class(TPersistent)
  private
    fLargeStep: TLineStep;
    fMiddleStep: TLineStep;
    fSmallStep: TLineStep;
    fOnStepChange: TNotifyEvent;
  protected
    procedure setLargeStep(const Value: TLineStep);
    procedure setMiddleStep(const Value: TLineStep);
    procedure setSmallStep(const Value: TLineStep);
    procedure StepChange(Sender: TObject);
  public
    constructor Create; virtual;
    destructor Destroy; override;
    property onStepChange: TNotifyEvent read fOnStepChange write fOnStepChange;
  published
    property SmallStep: TLineStep read fSmallStep write setSmallStep;
    property MiddleStep: TLineStep read fMiddleStep write setMiddleStep;
    property LargeStep: TLineStep read fLargeStep write setLargeStep;
  end;//TRulerSteps
//------------------------------------------------------------------------------

implementation

{ TLineStep }
//------------------------------------------------------------------------------
constructor TLineStep.Create(aOwner: TRulerSteps);
begin
  fOwner := aOwner;
  fOffset := 0;
  fLength := 2;
end;
//------------------------------------------------------------------------------
destructor TLineStep.Destroy;
begin
  inherited Destroy;
end;
//------------------------------------------------------------------------------
procedure TLineStep.Change;
begin
  Owner.StepChange(Self);
end;
//------------------------------------------------------------------------------
procedure TLineStep.doChange;
begin
  Change;
end;
//------------------------------------------------------------------------------
procedure TLineStep.setOffset(const Value: Integer);
begin
  if Value <> fOffset then begin
    fOffset := Value;
    doChange;
  end;
end;
//------------------------------------------------------------------------------
procedure TLineStep.setLength(const Value: Integer);
begin
  if Value <> fLength then begin
    fLength := Value;
    doChange;
  end;
end;
//------------------------------------------------------------------------------
procedure TLineStep.setColor(const Value: TColor);
begin
  if (Value <> fColor) then begin
    fColor := Value;
    doChange
  end;  
end;
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
{ TRulerSteps }
//------------------------------------------------------------------------------
constructor TRulerSteps.Create;
begin
  fLargeStep := TLineStep.Create(Self);
  fLargeStep.fOffset := 2;
  fLargeStep.fLength := 7;
  fLargeStep.Color := $00C8643C;
  fMiddleStep := TLineStep.Create(Self);
  fMiddleStep.fOffset := 3;
  fMiddleStep.fLength := 4;
  fMiddleStep.Color := $00C8643C;
  fSmallStep := TLineStep.Create(Self);
  fSmallStep.fOffset := 4;
  fSmallStep.fLength := 1;
  fSmallStep.Color := $00C8643C;
end;
//------------------------------------------------------------------------------
destructor TRulerSteps.Destroy;
begin
  fLargeStep.Free;
  fMiddleStep.Free;
  fSmallStep.Free;
  inherited Destroy;
end;
//------------------------------------------------------------------------------
procedure TRulerSteps.setLargeStep(const Value: TLineStep);
begin
  fLargeStep.Assign(Value);
end;
//------------------------------------------------------------------------------
procedure TRulerSteps.setMiddleStep(const Value: TLineStep);
begin
  fMiddleStep.Assign(Value);
end;
//------------------------------------------------------------------------------
procedure TRulerSteps.setSmallStep(const Value: TLineStep);
begin
  fSmallStep.Assign(Value);
end;
//------------------------------------------------------------------------------
procedure TRulerSteps.StepChange(Sender: TObject);
begin
  if Assigned(fOnStepChange) then
    fOnStepChange(Self);
end;
//------------------------------------------------------------------------------

end.




